import warnings
warnings.filterwarnings("ignore",".*GUI is implemented.*")
import numpy as np
import matplotlib.pyplot as plt

# Select data file
#import Tkinter as tk
#from Tkinter import ffiiledialog
#root = tk.Tk()
#root.withdraw()
#file_path = filedialog.askopenfilename()
import Tkinter, tkFileDialog
root = Tkinter.Tk()
root.withdraw()
file_path = tkFileDialog.askopenfilename()

# Read in the data
value , timestamp = np.loadtxt(file_path, unpack=True) #Check np identifier
global timeout
timeout=40000
value=value[value<timeout] #remove all timeouts so only decays remain

global T
T=max(value)

# Construct histogram

q75, q25 = np.percentile(value, [75 ,25])
iqr = q75 - q25
h_fd = round(2 * iqr * np.power(len(value),-1./3)) #Freedman Diaconis Rule of Thumb
flag=False;
# Prompt for histogram bin width
while not flag:
	h=input("Enter bin width (e.g. " + str(h_fd) + "):")
	#h=eval(h)
	flag=isinstance( h, int ) or isinstance( h , float )
	
edges=np.arange(0,T+h,h) # bin edges
count, edges=np.histogram(value,edges) # count data in bins
mids=edges[1:len(edges)]-edges[1]/2

pdf=count/float((sum(count)*h)) # convert from count to probability density
pdf_err=np.sqrt(count)/(sum(count)*h) # Root n approximate error bars

plt.ion() # interactive mode
plt.step(edges,np.append(pdf,0),where='post',color='blue') # step plot of histogram
plt.errorbar(mids,pdf,pdf_err,marker='None',linestyle='None',ecolor='blue' )

plt.yscale('log')
plt.xlabel('Decay Time (ns)')
plt.ylabel('Probability Density (ns$^{-1})$')
plt.xlim(0,T+h)
plt.pause(.0001) # Required to refresh plot in interactive mode

# Maximum Likelihood Estimation

x=np.arange(0,T,1)
y=np.zeros(len(x))
dist,=plt.plot(x,y,color='red')
# Muon Lifetime Prbability Density Function (Exponential plus constant background)
def lifetimepdf(param,x):
	tau , alpha  = param
	return (1-alpha)/(tau*(1-np.exp(-T/tau)))*np.exp(-x/tau)+alpha/T
def lifetimecdf(param,x):
	tau , alpha  = param
	return (1-alpha)/(1-np.exp(-T/tau))*(1-np.exp(-x/tau))+x*alpha/T
# Initial guesses at parameters
flag=False;
while not flag:
	flag_one=False
	while not flag_one:
		tau=input("Enter initial guess for muon lifetime in ns:")
		#tau=eval(tau)
		flag_one=isinstance( tau, int) or isinstance( tau , float)
	tau=float(tau);
	flag_two=False
	while not flag_two:
		alpha=input("Enter initial guess for noise fraction:")
		#alpha=eval(alpha)
		flag_two=isinstance( alpha, int ) or isinstance(alpha, float)
	alpha=float(alpha)
	y=lifetimepdf([tau , alpha],x)
	dist.remove()
	dist,=plt.plot(x,y,color='red')
	plt.pause(.0001)
	
	flag_three=False
	while not flag_three:
		happy=input("Are you happy with these initial values? (True/False)")
		#happy=eval(happy)
		flag_three=isinstance( happy, bool )
	
	flag=happy

# Log Likelihood function
def lnlike(param,x):
	return sum(np.log(lifetimepdf(param,x)))
import scipy.optimize as op

nll = lambda *args: -lnlike(*args)
bnds = ((0.0, timeout), (0.0, 1.0))
print('Fitting data')
result = op.minimize(nll, [tau , alpha], args=(value),method='TNC', bounds=bnds, tol=1e-10)
tau , alpha = result.x
y=lifetimepdf(result.x,x)
dist.remove()
dist,=plt.plot(x,y,color='red')
plt.pause(.0001)

# Estimate errors

dfactor=1e-3
tau_err =  -np.power(dfactor*tau,2)/( ( lnlike([tau*(1+dfactor) , alpha],value) - 2*lnlike([tau,alpha],value) + lnlike([tau*(1-dfactor) , alpha],value)) * np.sqrt( len(value) ))
alpha_err =  -np.power(dfactor*alpha,2)/( ( lnlike([tau , alpha*(1+dfactor)],value) - 2*lnlike([tau,alpha],value) + lnlike([tau , alpha*(1-dfactor)],value)) * np.sqrt( len(value) ))

print("Muon Lifetime & error:")
print([tau,tau_err])
print("Noise fraction & error:")
print([alpha,alpha_err])

# Chi square
from scipy.stats import chi2 , norm
expected=sum(count)*(lifetimecdf([tau,alpha],edges[1:len(edges)])-lifetimecdf([tau,alpha],edges[0:len(edges)-1]))
chisq=sum(np.power(count-expected,2)/expected)
print("Average Chi Squared Goodness of Fit and Null Hypothesis p-value")
print([chisq/len(mids),chi2.sf(chisq,sum(count!=0)-2)])

plt.show()
input("Press Enter to End")